28/12/97 13:53:48
   
transLibrary
    " Private - Returns the trans library."

    ^TransDLL32 current! 
allFilters
    "Private - Returns the filter specifications for all image files supported by the receiver.
	This method can be refined by subclasses..."

    ^ #(
        ('*.bmp;*.rle;*.dib' 'Bitmap Files (*.BMP,*.DIB,*.RLE)')
    )!  
fromBitmap: aBitmap
	"Build a transparent image from aBitmap."

    ^self fromBitmap: aBitmap transparentColor: nil!   
fromBitmap: aBitmap transparentColor: aColor
	"Build a transparent image from aBitmap using aColor as transparent color.
	If aColor isNil the leftTop pixel color is used as transparent."

    ^self new
		transparentColor: aColor;
		fromDIB: (DIB fromBitmap: aBitmap);
		yourself!
imageClass
    "Private - Returns the handle instance class"

    ^DIB!
fromDIB: aDIB transparentColor: aColor
	"Build a transparent image from aDIB using aColor as transparent color.
	If aColor isNil the leftTop pixel color is used as transparent."

    ^self new
		transparentColor: aColor;
		fromDIB: aDIB;
		yourself!  
fromDIB: aDIB
	"Build a transparent image from aDIB."

    ^self fromDIB: aDIB transparentColor: nil!  
bitmap
	"Returns the current receiver's bitmap."

	^hImage!
transLibrary
    " Private - Returns the trans library."

    ^self class transLibrary!
boundingBox
    "Returns the receiver's boundingBox."

    ^self bitmap boundingBox!   
bitmapInfo
    "Returns the receiver's bitmapInfo."

    ^self bitmap bitmapInfo!  
displayAt: aPoint with: aPen
    "Display the receiver with aPen at aPoint."

	| back fore boundingBox oldPalette |
	back := aPen backColor.
	fore := aPen foreColor.
	boundingBox := self boundingBox.
	aPen
		backColor: Color white;
		foreColor: Color black;
		yourself.

    self bitmap palette notNil ifTrue: [
        (oldPalette := GDILibrary selectPalette: aPen handle
            with: self bitmap palette asParameter
            forceBackground: false ) = 0
                ifTrue: [ ^self osError ].
        ( GDILibrary realizePalette: aPen handle ) = 16rFFFFFFFF
            ifTrue: [ ^self osError ] ].

	aPen
		copy: self mask pen
		from: boundingBox
		at: aPoint
		rule: 16r008800C6; "DSa"
		copy: self bitmap pen
		from: boundingBox
		at: aPoint
		rule: 16r00EE0086; "DSo"
		yourself.

	oldPalette notNil ifTrue: [
	    ( GDILibrary
	        selectPalette: aPen handle
	        with: oldPalette
	        forceBackground: false ) = 0
	            ifTrue: [ ^self osError ].
	].

	aPen
		backColor: back;
		foreColor: fore;
		yourself.
"
#define DSna 0x00220326L
#define DSa  0x008800C6L
#define DSo  0x00EE0086L
#define DSx  0x00660046L
"!   
initializeMask
	" Private - Initializes the receiver's mask"

	mask := self monochromeMask.
		"Use color masks for better perfomance (default).
		Force monochrome masks for low memory requirements."!  
width
    "Returns the receiver's width"

    self isValid ifFalse: [^0].
    ^self boundingBox width!
height
    "Returns the receiver's height"

    self isValid ifFalse: [^0].
    ^self boundingBox height! 
transparentDIBFrom: aDIB
    " Private - Build a transparent dib from aDIB.
	Returns nil if failed. "

    | bitmapInfo rleDIB rleBitmapInfo rleBits result |
    bitmapInfo := aDIB bitmapInfo.
    bitmapInfo compression ~= 0 ifTrue: [  ^nil ].
    rleBitmapInfo := aDIB bitmapInfo copy.
    "we must copy internal structure if it's in externalMemory."
    rleBitmapInfo contents isExternalAddress
    ifTrue: [
        rleBitmapInfo contents:
            (rleBitmapInfo contents
                copyFrom: 1
                to: rleBitmapInfo biSize"contents memorySize")
    ].
	rleBits := ByteArray new: aDIB bits size * 2. "Allocate enought space"
	result := self transLibrary
		rleDib: rleBitmapInfo asParameter
		bits: aDIB bits asParameter
		color: self transparentColor
		resultBits: rleBits.
	result = 0 ifTrue: [ "API failed." ^nil ].
	rleDIB := DIB new.
	rleDIB
		bitmapInfo: rleBitmapInfo;
		bits: (rleBits copyFrom: 1 to: rleBitmapInfo sizeImage).
	^rleDIB!
palette
	"Returns the current receiver's palette."

	^self bitmap palette! 
extent
    "Returns the receiver's extent."

    ^self bitmap extent!  
release
	"Release the receiver's resources..."

	self
		mask: nil;
		privateHandle: nil;
		yourself.
    ^super release!   
mask: aBitmap
    "Sets the receiver's mask."

	mask release.
    mask := aBitmap.!   
initializeTransparentColor
	" Private - Initializes the receiver's transparentColor"

	transparentColor := -1. "use leftTop pixel color as transparent color."!
mask
    "Returns the instance var of the receiver
     for external manipulation"

	mask isNil ifTrue: [ self initializeMask ].
    ^mask!  
fileInFrom: aPathName format: aFormat
    "Private - Read the receiver internal structure from aFile.
    aFormat will be ignored."

	self pathName: aPathName.
    self fromDIB: (DIB fromFile: aPathName)! 
monochromeMask
	"Rerturns a valid monochrome mask for the receiver's display."

	| bitmap hPalette result handle |
	bitmap := self bitmap.
	hPalette := bitmap createDIBPalette.
	handle := self transLibrary
		makeMask: bitmap asParameter
		palette: hPalette asParameter
		colorRef: self transparentColor.
	result := Bitmap fromHandle: handle.
	hPalette release.
	^result!
transparentColor: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    transparentColor := anObject! 
transparentColor
    "Returns the instance var of the receiver
     for external manipulation"

	transparentColor isNil ifTrue: [ self initializeTransparentColor ].
    ^transparentColor!  
fromDIB: aDIB
    " Build the receiver's transparent dib from aDIB. "

    | rleDIB |
	rleDIB := self transparentDIBFrom: aDIB.
	rleDIB isNil ifTrue: [ rleDIB := aDIB ].
	self privateHandle: rleDIB asBitmap.

	self mask: rleDIB asBitmap.
	self monochromeMask
		displayAt: 0@0
		with: self mask pen.!
fileName
    "Private - Answer the receiver's file name."

     ^'Trans.Dll'
    "Debug 'C:\OStyle\AleIsWorkingHere\Future Projects\Trans\Trans\Debug\Trans.dll' "
    "Release 'Trans.Dll' "!   
trueMaskBlt: hdc
	x: x y: y width: dx height: dy
	source: hdcS mask: hdcMask
	xSrc: x0 ySrc: y0
	"Use this API for a true mask blt."

    <api: TrueMaskBlt ulong
		long long long long
		ulong ulong
		long long
		none>
    ^self invalidArgument
"
void TrueMaskBlt (HDC hdcD,int x,int y,int dx,int dy,HDC hdcS,HDC hdcMask,int x0,int y0);
"!  
makeMask: hBitmap palette: hPal colorRef: rgb
	"Builds a transparent mask that you can use to draw (using TransMaskBlt) the given bitmap causing the given color to be transparent.
	Returns the transparent mask's bitmap handle."

    <api: MakeTransMask ulong ulong ulong ulongReturn>
    ^self invalidArgument
"
HBITMAP MakeTransMask(HBITMAP hbm, HPALETTE hpal, COLORREF rgb);
"!   
rleDib: resultBitmapInfo bits: bits color: uColor resultBits: resultBits
    "Use this API for building a transparent (rle) dib in memory.
    Given a DIB make a RLE out of it, treating a passed color as transparent.
    The output buffer must be big enougth to hold the RLE, or bad things will happen...
    Returns the resulting Rle bits size.
    WARNING: resultBitmapInfo & resultBits will be modified.
    At entry resultBitmapInfo MUST have been set with source bitmapInfo and will be modified to contain destination Rle bitmapInfo."

    <api: RleDib struct struct ulong struct ulongReturn>
    ^self invalidArgument
"
long RleDib(
    LPBITMAPINFOHEADER lpbi,    // Input/Output: bitmap info to compress.
    LPVOID             pDib,    // Input: DIB bits to compress.
    UINT           uColor,    // Input: trasnparent color (-1 for none)
    LPVOID             pRle)    // Output: RLE bits output
    // Returns Rle bits size (or zero if a problem has been encountered -e.g. bitmap yet compressed-)
"! 
bitBlt: hdcDest
	x: x y: y width: dx height: dy
	srcDC: hdcSource
	xSrc: x0 ySrc: y0
	"Transparent bitblt that uses the current background color of the DC as the transparent color."

    <api: TransBlt ulong long long long long ulong long long boolean>
    ^self invalidArgument
"
BOOL TransBlt(HDC hdcD,int x,int y,int dx,int dy,HDC hdcS,int x0,int y0);
"!   
stretchDIBits: hdcDest
	x: x y: y width: dx height: dy
	srcX: sx srcY: sy srcWidth: sdx srcHeight: sdy
	bits: lpBits bmi: lpbi
	colorUse: wUsage rop: rop
	"Draw a DIB bitmap treating the background color of the DC as transparent."

    <api: TransStretchDIBits ulong
		long long long long
		long long long long
		struct struct
		ushort ulong
		long
	>
    ^self invalidArgument
"
int TransStretchDIBits(HDC hdc,
                       WORD DstX, WORD DstY, WORD DstDX, WORD DstDY,
                       WORD SrcX, WORD SrcY, WORD SrcDX, WORD SrcDY,
                       LPVOID lpBits, LPBITMAPINFO lpbi,
		       WORD wUsage, DWORD rop);
"!  
maskBlt: hdc
    x: x y: y width: dx height: dy
    source: hdcS mask: hdcMask
    xSrc: x0 ySrc: y0
    "Use this API for a mask blt where the source (hdcS) has black (ie 0) every where the mask has white (ie 1)"

    <api: TransMaskBlt ulong
        long long long long
        ulong ulong
        long long
        none>
    ^self invalidArgument
"
void TransMaskBlt (HDC hdcD,int x,int y,int dx,int dy,HDC hdcS,HDC hdcMask,int x0,int y0);
"!